/* commands.c
    Routines for the standard FTP commands.
    Copyright (C) 1999 Eric Gillespie, Jr.

    This file is part of dpsftplib.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,  as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>

#include "dpsftplib.h"
#include "ftpsite.h"

gboolean
ftpNoop(ftpSite *site)
{
	g_return_val_if_fail(site != NULL, FALSE);

	if (!ftpCommand(site, "NOOP\r\n")) {
		ftperrno = FTP_LOSTCONN;
		return FALSE;
	}

	if (ftpGetReply(site)[0] != '2') {
		ftperrno = FTP_NOOPFAIL;
		return FALSE;
	}

	return TRUE;
}

gboolean
ftpCdup(ftpSite *site)
{
	g_return_val_if_fail(site != NULL, FALSE);

	if (!ftpCommand(site, "CDUP\r\n")) {
		ftperrno = FTP_LOSTCONN;
		return FALSE;
	}

	if (ftpGetReply(site)[0] != '2') {
		ftperrno = FTP_RCHDIRFAIL;
		return FALSE;
	}

	return ftpPwd(site);
}

gboolean
ftpChdir(ftpSite *site, const gchar *dir)
{
	g_return_val_if_fail(site != NULL, FALSE);
	g_return_val_if_fail(dir != NULL, FALSE);

	if (strcmp(dir, site->pwd) == 0)
		return TRUE;

	if (!ftpCommand(site, "CWD %s\r\n", dir)) {
		ftperrno = FTP_LOSTCONN;
		return FALSE;
	}
	if (ftpGetReply(site)[0] != '2') {
		ftperrno = FTP_RCHDIRFAIL;
		return FALSE;
	}

	return ftpPwd(site);
}

gboolean
ftpType(ftpSite *site, const gint type)
{
	g_return_val_if_fail(site != NULL, FALSE);
	g_return_val_if_fail((type == 'I') || (type == 'A'), FALSE);

	if (!site->connected) {
		ftperrno = FTP_NOTCONNECTED;
		return FALSE;
	}

	if (site->type == type)
		return TRUE;

	if (!ftpCommand(site, "TYPE %c\r\n", type)) {
		ftperrno = FTP_LOSTCONN;
		return FALSE;
	}

	if (ftpGetReply(site)[0] != '2') {
		ftperrno = FTP_TYPEFAIL;
		return FALSE;
	}

	site->type = type;

	return TRUE;
}

gboolean
ftpPwd(ftpSite *site)
{
	gint i = 0;

	g_return_val_if_fail(site != NULL, FALSE);

	if (site->pwd)
		g_free(site->pwd);

	if (!ftpCommand(site, "PWD\r\n")) {
		ftperrno = FTP_LOSTCONN;
		return FALSE;
	}

	site->pwd = g_strdup(ftpGetReply(site) + 5);

	while (site->pwd[++i] != '"')
		; /* do nothing */

	site->pwd[i] = '\0';

	return TRUE;
}
