/* ftpdirectory.c
    Routines to manage the ftpDirectory data type.
    Copyright (C) 1999 Eric Gillespie, Jr.

    This file is part of dpsftplib.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,  as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>

#include "dpsftplib.h"
#include "ftpdirectory.h"

ftpDirectory *
ftpDirectoryNew(const gchar *name)
{
	ftpDirectory *directory;

	g_return_val_if_fail(name != NULL, NULL);

	ftperrno = FTP_SYS;

	directory = g_malloc(sizeof(ftpDirectory));
	if (!directory)
		return NULL;
	else
		directory->name = NULL;

	directory->name = g_malloc(strlen(name) + 1);
	if (!directory->name) {
		ftpDirectoryDelete(directory);
		return NULL;
	} else
		strcpy(directory->name, name);

	directory->dirlist = directory->filelist = NULL;

	return directory;
}

void
ftpDirectoryDelete(ftpDirectory *directory)
{
	g_return_if_fail(directory != NULL);

	if (directory->name)
		g_free(directory->name);

	g_list_foreach(directory->dirlist, (GFunc) ftpFileDelete, NULL);
	g_list_free(directory->dirlist);
	g_list_free(directory->filelist);

	g_free(directory);
}

const gchar *
ftpDirectoryGetName(const ftpDirectory *dir)
{
	g_return_val_if_fail(dir != NULL, NULL);

	return dir->name;
}

const GList *
ftpDirectoryGetDirList(const ftpDirectory *dir)
{
	g_return_val_if_fail(dir != NULL, NULL);

	return dir->dirlist;
}

const GList *
ftpDirectoryGetFileList(const ftpDirectory *dir)
{
	g_return_val_if_fail(dir != NULL, NULL);

	return dir->filelist;
}
