/* sockio.c
    Functions for communicating through a BSD socket.
    Copyright (C) 1999  Eric Gillespie, Jr.

    This file is part of dpsftplib.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,  as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>

#include "dpsftplib.h"

int
OpenSocket(char *hostname, int port)
{
	int skt = 0;
	struct sockaddr_in sktin;
	struct hostent *host;

	if ( (skt = socket(PF_INET, SOCK_STREAM, 0)) < 0)
		return -1;

	memset(&sktin, 0, sizeof(sktin));
	sktin.sin_family = AF_INET;

	sktin.sin_port = htons(port);
	if (!inet_aton(hostname, &sktin.sin_addr)) {
		if ((host = gethostbyname(hostname)))
			memcpy(&sktin.sin_addr, host->h_addr, host->h_length);
		else
			return -1;
	}

	if (connect(skt, (struct sockaddr*)&sktin, sizeof(sktin)) < 0)
		return -1;

	return skt;
}

/*
** FileSocket
**
** Modified example from the UNIX Socket FAQ (http://www.ibrado.com/sock-faq/).
**
** Create a stdio stream from a socket
**
** The stream is unidirectional; dup() the socket and create 2 streams if
** you want bidirectionality.
**
** Modes:
**   r   default read mode
**   w   default write mode
**   rt  text read mode (line-buffered)
**   wt  text write mode (line-buffered)
**
** No translation of data is implied by the references to 'text' & 'binary'
*/

/* buffers less than 4k are inefficient on many implementations */
/* 28k writes to a 56k socket buffer is a magic combination on some */
/* high-speed networks, and should give near-optimal throughput on */
/* almost any system */

#define SOCK_SMALLBUF (4*1024)
#define SOCK_LARGEBUF (28*1024)

FILE *
FileSocket(int skt, char *mode)
{
	int bufsz = SOCK_SMALLBUF;
	int bufmode = _IOLBF;
	FILE *fp;

	fp = fdopen(skt, (mode[0] == 'w') ? "w" : "r");
	if (!fp)
		return NULL;

	if (mode[1] != 't') {
		bufmode = _IOFBF;
		bufsz = SOCK_LARGEBUF * 2;
		setsockopt(skt, SOL_SOCKET,
			   (mode[0] == 'w') ? SO_SNDBUF : SO_RCVBUF,
			   &bufsz, sizeof(bufsz));
		bufsz = SOCK_LARGEBUF;
	}
	setvbuf(fp, NULL, bufmode, bufsz);

	return fp;
}
