/* util.c
    Utility functions used internally by the library, perhaps useful to a user.
    Copyright (C) 1999 Eric Gillespie, Jr.

    This file is part of dpsftplib.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,  as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include "dpsftplib.h"
gint errno;
guint32
ftpRemoteLinkIsFile(ftpSite *site, const gchar *link)
{
	const gchar *r;

	g_return_val_if_fail(site != NULL, -1);
	g_return_val_if_fail(link != NULL, -1);

	if (!ftpCommand(site, "SIZE %s\r\n", link)) {
		ftperrno = FTP_LOSTCONN;
		return -1;
	}

	r = ftpGetReply(site);

	if (!r)
		return -1;

	if (r[0] != '2')
		return 0;

	return strtol(r + 4, NULL, 10);
}

gboolean
ftpEatSlashes(gchar *s, gint nbytes)
{
        gchar *new;
        gint i, j;

        new = g_malloc(strlen(s) + 1);
        if (!new)
		return FALSE;

        j = 0;
        for (i = 0; i <= strlen(s); i++) {
                new[j++] = s[i];
                if (s[i] == '/') {
                        while (s[++i] == '/');
                        i--;
                }
        }
        new[j - 1] = '\0';

        strncpy(s, new, nbytes);
	g_free(new);

        return TRUE;
}

gchar *
ftpParseEnv(gchar *in)
{
        const gchar separators[] = " \t\n\r`~!@#$%^&*()-_=+\\|[{]};:'\",<.>/?";
	GString *out, *var;
	gchar *val, *outstr, *tmp, *sub;
	gint n, len, lentmp, i;
	gboolean found;

	out = g_string_new(NULL);
	var = g_string_new(NULL);
	len = strlen(in);

	sub = in;
	while ((sub = strchr(sub, '$'))) {
		n = sub - in;
		if (sub > in) {
			if ((sub - 1)[0] == '\\') {
				n--;
				tmp = g_malloc(n + 1);
				memset(tmp, 0, n + 1);
				strncpy(tmp, in, n);
				out = g_string_append(out, tmp);
				g_free(tmp);
				in += n + 2;
				sub++;
				out = g_string_append_c(out, '$');
				continue;
			}
		}
		tmp = g_malloc(n + 1);
		memset(tmp, 0, n + 1);
		strncpy(tmp, in, n);
		out = g_string_append(out, tmp);
		g_free(tmp);

		sub++;
		n++;
		in += n;
		len -= n;

		var = g_string_truncate(var, 0);
		if (sub[0] == '{') {
			sub++;
			in++;
			len++;

			found = FALSE;
			for (i = 0; i < len; i++) {
				if (sub[i] == '}')
					break;

				var = g_string_append_c(var, sub[i]);
			}
			lentmp = strlen(var->str) + 1;
		} else {
			for (i = 0; i < len; i++) {
				if (strchr(separators, sub[i]))
					break;

				var = g_string_append_c(var, sub[i]);
			}
			lentmp = strlen(var->str);
		}

		val = g_getenv(var->str);
		if (val)
			out = g_string_append(out, val);

		in += lentmp;
		len -= lentmp;
	}
	out = g_string_append(out, in);

	outstr = out->str;
	g_string_free(out, FALSE);
	g_string_free(var, TRUE);

	return outstr;
}

gint
ftpLocalFileExists(const char *fn)
{
	gint fd;

	g_return_val_if_fail(fn != NULL, -1);

	fd = open(fn, O_RDONLY);
	if (fd < 0) {
		if (errno == ENOENT)
			return 0;
		else
			return -1;
	}
	close(fd);

	return 1;
}
